import { Context, FetchOptions, ContentRange } from '../typings';
export declare class HttpError extends Error {
    statusCode: number;
    status: number;
    body: unknown;
    constructor(statusCode: number | undefined, { body, message }: {
        body?: unknown;
        message?: string;
    });
}
declare type BuildFetchUrlOptions = {
    url: string;
    urlParams: Record<string, {
        isQueryParamValue?: boolean;
    }>;
    stripTrailingSlashes?: boolean;
};
export declare const buildFetchUrl: (context: Context, { url, urlParams, stripTrailingSlashes }: BuildFetchUrlOptions) => string;
export declare const buildFetchOpts: (context: Context, { method, headers, credentials, query, body }: FetchOptions) => FetchOptions;
export declare const parseResponseBody: <T = unknown>(res: Response) => Promise<T>;
export interface SerializableResponse<T = unknown> {
    body: T;
    code: Response['status'];
    payload: Pick<Response, 'status' | 'ok'> & {
        headers: Record<string, string>;
        [s: string]: unknown;
    };
    contentRange?: ContentRange | null;
    [s: string]: unknown;
}
export declare const serializeResponse: <T = unknown>(res: Response) => Promise<SerializableResponse<T>>;
declare const fetch: (url: string, options?: FetchOptions) => Promise<Response>;
export default fetch;
