"use strict";

var _interopRequireDefault = require("@babel/runtime/helpers/interopRequireDefault");

Object.defineProperty(exports, "__esModule", {
  value: true
});
Object.defineProperty(exports, "initialState", {
  enumerable: true,
  get: function get() {
    return _defaults.initialState;
  }
});
exports.createRootReducer = exports.createReducers = exports.createReducer = void 0;

var _toConsumableArray2 = _interopRequireDefault(require("@babel/runtime/helpers/toConsumableArray"));

var _defineProperty2 = _interopRequireDefault(require("@babel/runtime/helpers/defineProperty"));

var _lodash = require("lodash");

var _defaults = require("../defaults");

var _util = require("../helpers/util");

var _types = require("../types");

function ownKeys(object, enumerableOnly) { var keys = Object.keys(object); if (Object.getOwnPropertySymbols) { var symbols = Object.getOwnPropertySymbols(object); if (enumerableOnly) symbols = symbols.filter(function (sym) { return Object.getOwnPropertyDescriptor(object, sym).enumerable; }); keys.push.apply(keys, symbols); } return keys; }

function _objectSpread(target) { for (var i = 1; i < arguments.length; i++) { var source = arguments[i] != null ? arguments[i] : {}; if (i % 2) { ownKeys(Object(source), true).forEach(function (key) { (0, _defineProperty2["default"])(target, key, source[key]); }); } else if (Object.getOwnPropertyDescriptors) { Object.defineProperties(target, Object.getOwnPropertyDescriptors(source)); } else { ownKeys(Object(source)).forEach(function (key) { Object.defineProperty(target, key, Object.getOwnPropertyDescriptor(source, key)); }); } } return target; }

var getUpdateArrayData = function getUpdateArrayData(action, itemId) {
  var actionOpts = action.options || {};
  var idKey = (0, _util.getIdKey)(action, {
    multi: false
  });

  if (!(0, _lodash.isPlainObject)(action.context)) {
    return {};
  }

  var actionContext = action.context;
  return actionOpts.assignResponse ? (0, _util.find)(action.body, (0, _defineProperty2["default"])({}, idKey, itemId)) : Object.keys(actionContext).reduce(function (soFar, key) {
    if (key !== 'ids') {
      soFar[key] = actionContext[key];
    }

    return soFar;
  }, {});
};

var createDefaultReducers = function createDefaultReducers(reduceOptions) {
  var initialState = _defaults.initialState;
  var mergeItem = reduceOptions.mergeItem || _defaults.defaultMergeItem;
  return {
    create: function create() {
      var state = arguments.length > 0 && arguments[0] !== undefined ? arguments[0] : initialState;
      var action = arguments.length > 1 ? arguments[1] : undefined;
      var actionOpts = action.options || {};

      switch (action.status) {
        case 'pending':
          // @TODO optimistic updates?
          return _objectSpread(_objectSpread({}, state), {}, {
            isCreating: true
          });

        case 'resolved':
          {
            var nextState = {};

            if (actionOpts.assignResponse) {
              var createdItem = action.body;
              nextState.items = (state.items || []).concat(createdItem);
            }

            return _objectSpread(_objectSpread({}, state), {}, {
              isCreating: false
            }, nextState);
          }

        case 'rejected':
          return _objectSpread(_objectSpread({}, state), {}, {
            isCreating: false
          });

        default:
          return state;
      }
    },
    fetch: function fetch() {
      var state = arguments.length > 0 && arguments[0] !== undefined ? arguments[0] : initialState;
      var action = arguments.length > 1 ? arguments[1] : undefined;

      switch (action.status) {
        case 'pending':
          {
            var actionOpts = action.options || {};
            var didInvalidate = !!actionOpts.invalidateState;
            return _objectSpread(_objectSpread({}, state), {}, {
              isFetching: true,
              didInvalidate: didInvalidate
            }, didInvalidate ? {
              items: []
            } : {});
          }

        case 'resolved':
          {
            var nextState = {};
            nextState.items = action.body;
            var contentRange = action.contentRange;
            var isPartialContent = action.code === 206;

            if (isPartialContent && contentRange && contentRange.first > 0) {
              nextState.items = state.items.slice(0, contentRange.first).concat(nextState.items).concat(state.items.slice(contentRange.last, state.items.length));
            }

            return _objectSpread(_objectSpread({}, state), {}, {
              isFetching: false,
              didInvalidate: false,
              lastUpdated: action.receivedAt
            }, nextState);
          }

        case 'rejected':
          return _objectSpread(_objectSpread({}, state), {}, {
            isFetching: false
          });

        default:
          return state;
      }
    },
    get: function get() {
      var state = arguments.length > 0 && arguments[0] !== undefined ? arguments[0] : initialState;
      var action = arguments.length > 1 ? arguments[1] : undefined;
      var actionOpts = action.options || {};

      switch (action.status) {
        case 'pending':
          {
            var idKey = (0, _util.getIdKey)(action, {
              multi: false
            });
            var id = (0, _util.isObject)(action.context) ? action.context[idKey] : action.context;
            var hasConflictingContext = id && state.item ? state.item[idKey] !== id : false;
            var didInvalidate = !!actionOpts.invalidateState || hasConflictingContext;
            return _objectSpread(_objectSpread({}, state), {}, {
              isFetchingItem: true,
              didInvalidateItem: didInvalidate
            }, didInvalidate ? {
              item: null
            } : {});
          }

        case 'resolved':
          {
            var partialItem = action.body;
            var nextItem = actionOpts.mergeResponse ? mergeItem(state.item, partialItem) : partialItem;
            var nextState = {
              item: nextItem
            };

            if (actionOpts.assignResponse) {
              var _idKey = (0, _util.getIdKey)(action, {
                multi: false
              });

              var prevListItemIndex = state.items.findIndex(function (el) {
                return el[_idKey] === partialItem[_idKey];
              });

              if (prevListItemIndex !== -1) {
                var prevListItem = state.items[prevListItemIndex];
                var nextListItem = actionOpts.mergeResponse ? mergeItem(prevListItem, partialItem) : partialItem;
                state.items.splice(prevListItemIndex, 1, nextListItem);
                nextState.items = state.items.slice();
              }
            }

            return _objectSpread(_objectSpread({}, state), {}, {
              isFetchingItem: false,
              didInvalidateItem: false,
              lastUpdatedItem: action.receivedAt
            }, nextState);
          }

        case 'rejected':
          return _objectSpread(_objectSpread({}, state), {}, {
            isFetchingItem: false
          });

        default:
          return state;
      }
    },
    update: function update() {
      var state = arguments.length > 0 && arguments[0] !== undefined ? arguments[0] : initialState;
      var action = arguments.length > 1 ? arguments[1] : undefined;

      switch (action.status) {
        case 'pending':
          // Update object in store as soon as possible?
          return _objectSpread(_objectSpread({}, state), {}, {
            isUpdating: true
          });

        case 'resolved':
          {
            // Assign context or returned object
            var idKey = (0, _util.getIdKey)(action, {
              multi: false
            });
            var id = (0, _util.isObject)(action.context) ? action.context[idKey] : action.context;
            var actionOpts = action.options || {};
            var update = actionOpts.assignResponse ? action.body : action.context;
            var listItemIndex = state.items.findIndex(function (el) {
              return el[idKey] === id;
            });
            var updatedItems = state.items.slice();

            if (listItemIndex !== -1) {
              updatedItems[listItemIndex] = _objectSpread(_objectSpread({}, updatedItems[listItemIndex]), update);
            }

            var updatedItem = state.item && state.item[idKey] === id ? _objectSpread(_objectSpread({}, state.item), update) : state.item;
            return _objectSpread(_objectSpread({}, state), {}, {
              isUpdating: false,
              items: updatedItems,
              item: updatedItem
            });
          }

        case 'rejected':
          return _objectSpread(_objectSpread({}, state), {}, {
            isUpdating: false
          });

        default:
          return state;
      }
    },
    updateMany: function updateMany() {
      var state = arguments.length > 0 && arguments[0] !== undefined ? arguments[0] : initialState;
      var action = arguments.length > 1 ? arguments[1] : undefined;

      switch (action.status) {
        case 'pending':
          // Update object in store as soon as possible?
          return _objectSpread(_objectSpread({}, state), {}, {
            isUpdatingMany: true
          });

        case 'resolved':
          {
            // Assign context or returned object
            var actionOpts = action.options || {};
            var idKey = (0, _util.getIdKey)(action, {
              multi: false
            });
            var idKeyMulti = (0, _util.getIdKey)(action, {
              multi: true
            });

            var _ref = actionOpts.query || action.context,
                ids = _ref[idKeyMulti];

            var updatedItems = state.items.map(function (item) {
              if (!ids || ids.includes(item[idKey])) {
                var _updatedItem = getUpdateArrayData(action, item[idKey]);

                return _updatedItem ? _objectSpread(_objectSpread({}, item), _updatedItem) : item;
              }

              return item;
            }); // Also impact state.item? (@TODO opt-in/defautl?)

            var updatedItem = state.item && (!ids || ids.includes(state.item[idKey])) ? _objectSpread(_objectSpread({}, state.item), getUpdateArrayData(action, state.item[idKey])) : state.item;
            return _objectSpread(_objectSpread({}, state), {}, {
              isUpdatingMany: false,
              items: updatedItems,
              item: updatedItem
            });
          }

        case 'rejected':
          return _objectSpread(_objectSpread({}, state), {}, {
            isUpdatingMany: false
          });

        default:
          return state;
      }
    },
    "delete": function _delete() {
      var state = arguments.length > 0 && arguments[0] !== undefined ? arguments[0] : initialState;
      var action = arguments.length > 1 ? arguments[1] : undefined;

      switch (action.status) {
        case 'pending':
          // Update object in store as soon as possible?
          return _objectSpread(_objectSpread({}, state), {}, {
            isDeleting: true
          });

        case 'resolved':
          {
            // @NOTE Do not update items array when an empty context was provided
            // Can happen with custom resource not using id params
            if (!action.context) {
              return _objectSpread(_objectSpread({}, state), {}, {
                isDeleting: false
              });
            }

            var idKey = (0, _util.getIdKey)(action, {
              multi: false
            });
            var id = action.context[idKey] || action.context;
            return _objectSpread(_objectSpread({}, state), {}, {
              isDeleting: false,
              items: (0, _toConsumableArray2["default"])(state.items.filter(function (el) {
                return el[idKey] !== id;
              }))
            });
          }

        case 'rejected':
          return _objectSpread(_objectSpread({}, state), {}, {
            isDeleting: false
          });

        default:
          return state;
      }
    },
    deleteMany: function deleteMany() {
      var state = arguments.length > 0 && arguments[0] !== undefined ? arguments[0] : initialState;
      var action = arguments.length > 1 ? arguments[1] : undefined;

      switch (action.status) {
        case 'pending':
          // Update object in store as soon as possible?
          return _objectSpread(_objectSpread({}, state), {}, {
            isDeletingMany: true
          });

        case 'resolved':
          {
            var actionOpts = action.options || {};
            var idKey = (0, _util.getIdKey)(action, {
              multi: false
            });
            var idKeyMulti = (0, _util.getIdKey)(action, {
              multi: true
            });

            var _ref2 = actionOpts.query || action.context,
                ids = _ref2[idKeyMulti];

            if (!ids) {
              return _objectSpread(_objectSpread({}, state), {}, {
                isDeletingMany: false,
                items: [],
                item: null
              });
            }

            return _objectSpread(_objectSpread({}, state), {}, {
              isDeletingMany: false,
              items: (0, _toConsumableArray2["default"])(state.items.filter(function (el) {
                return !ids.includes(el[idKey]);
              })),
              item: state.item && ids.includes(state.item[idKey]) ? null : state.item
            });
          }

        case 'rejected':
          return _objectSpread(_objectSpread({}, state), {}, {
            isDeletingMany: false
          });

        default:
          return state;
      }
    }
  };
};

var createReducer = function createReducer(actionId, reduceOptions) {
  var defaultReducers = arguments.length > 2 && arguments[2] !== undefined ? arguments[2] : createDefaultReducers(reduceOptions);

  // Custom reducers
  if (reduceOptions.reduce && (0, _util.isFunction)(reduceOptions.reduce)) {
    return reduceOptions.reduce;
  } // Do require a custom reduce function for pure actions


  if (reduceOptions.isPure) {
    throw new Error("Missing `reduce` option for pure action `".concat(actionId, "`"));
  } // Default reducers


  if (defaultReducers[actionId]) {
    return defaultReducers[actionId];
  } // Custom actions


  var gerundName = reduceOptions.gerundName || (0, _util.getGerundName)(actionId);
  var gerundStateKey = "is".concat((0, _util.ucfirst)(gerundName));
  return function () {
    var state = arguments.length > 0 && arguments[0] !== undefined ? arguments[0] : _defaults.initialState;
    var action = arguments.length > 1 ? arguments[1] : undefined;

    switch (action.status) {
      case 'pending':
        // Update object in store as soon as possible?
        return _objectSpread(_objectSpread({}, state), {}, (0, _defineProperty2["default"])({}, gerundStateKey, true));

      case 'resolved':
        // eslint-disable-line
        return _objectSpread(_objectSpread({}, state), {}, (0, _defineProperty2["default"])({}, gerundStateKey, false));

      case 'rejected':
        return _objectSpread(_objectSpread({}, state), {}, (0, _defineProperty2["default"])({}, gerundStateKey, false));

      default:
        return state;
    }
  };
};

exports.createReducer = createReducer;

var createReducers = function createReducers() {
  var actions = arguments.length > 0 && arguments[0] !== undefined ? arguments[0] : {};
  var reduceOptions = arguments.length > 1 && arguments[1] !== undefined ? arguments[1] : {};
  var actionKeys = Object.keys(actions);
  var defaultReducers = createDefaultReducers(reduceOptions);
  return actionKeys.reduce(function (actionReducers, actionId) {
    // d(omit(actions[actionId], SUPPORTED_REDUCE_OPTS));
    var combinedReduceOptions = _objectSpread(_objectSpread({}, reduceOptions), actions[actionId]);

    var reducerKey = (0, _types.getActionType)(actionId).toLowerCase();
    actionReducers[reducerKey] = createReducer(actionId, combinedReduceOptions, defaultReducers);
    return actionReducers;
  }, {});
};

exports.createReducers = createReducers;

var createRootReducer = function createRootReducer() {
  var reducers = arguments.length > 0 && arguments[0] !== undefined ? arguments[0] : {};

  var _ref3 = arguments.length > 1 ? arguments[1] : undefined,
      resourceName = _ref3.resourceName,
      _ref3$scope = _ref3.scope,
      scope = _ref3$scope === void 0 ? (0, _types.getTypesScope)(resourceName) : _ref3$scope;

  var scopeNamespace = scope ? "".concat(scope, "/") : '';

  var rootReducer = function rootReducer() {
    var state = arguments.length > 0 && arguments[0] !== undefined ? arguments[0] : _objectSpread({}, _defaults.initialState);
    var action = arguments.length > 1 ? arguments[1] : undefined;

    // Only process relevant namespace
    if (scopeNamespace && !String(action.type).startsWith(scopeNamespace)) {
      return state;
    } // Only process relevant action type


    var type = action.type.substr(scopeNamespace.length).toLowerCase(); // Check for a matching reducer

    if (reducers[type]) {
      return reducers[type](state, action);
    }

    return state;
  };

  return rootReducer;
};

exports.createRootReducer = createRootReducer;
//# sourceMappingURL=index.js.map